/*=============================================================================
vitalnix/dxt - User Management UI
init.c - Init process
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 1999 - 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
//#include <libxml/parser.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <popt.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <libHX.h>
//#include <libHXtxd.h>
#include "dxt-src/global.h"

static int count_args(const char **);

//-----------------------------------------------------------------------------
void exit_pg(signed int v, char *fmt, ...) {
  //HXtxd_dev_deinit(&_textcon);
  if(fmt != NULL) {
    if(v & P_ERR) { perror(fmt); }
    else {
      va_list argp;
      va_start(argp, fmt);
      vfprintf(stderr, fmt, argp);
      va_end(argp);
    }
  }
  exit(v & ~P_ERR);
  return;
}

int iabs(signed int p) {
  if(p < 0) { p = -p; }
  return p;
}

void init_applock(void) {
  char buf[16];
  int fd;

  if((fd = open(Optmap.f_lock, O_RDWR | O_CREAT, 0666)) < 0) {
    exit_pg(1 | P_ERR, "open()");
  }

  if(lockf(fd, F_TLOCK, 0) < 0) {
    exit_pg(1 | P_ERR, "%s already running?\nlockf()", Optmap.progn);
  }

  ftruncate(fd, 0);
  snprintf(buf, 8, "%d\n", getpid());
  write(fd, buf, strlen(buf));
  fcntl(fd, F_SETFD, FD_CLOEXEC);
  // never close that file or the lock will be gone
  return;
}

// XXX obsolete it.
/*static void GSPEC1(xmlNode *ptr, void *P_tagname, void *P_target) {
  // general spec
  void *pa;
  if(strcmp(ptr->name, P_tagname) == 0 &&
   (pa = xmlGetProp(ptr, "val")) != NULL) {
    HX_strclone(P_target, pa);
    xmlFree(pa);
  }
}

static void PSPEC1(xmlNode *ptr, void *P_tagname, void *P_target, void *P_args) {
  // prog spec
  void *pa;
  if(strcmp(ptr->name, P_tagname) == 0 &&
   (pa = xmlGetProp(ptr, "prog")) != NULL) {
    HX_strclone(P_target, pa);
    xmlFree(pa);
    if((pa = xmlGetProp(ptr, "args")) != NULL) {
      HX_strclone(P_args, pa);
      xmlFree(pa);
    }
  }
}

int load_config(char *fn) {
  char *pa;
  xmlDoc *doc;
  xmlNode *ptr;
  xmlKeepBlanksDefault(0);
  if((doc = xmlParseFile(fn)) == NULL) { return 0; }
  if((ptr = xmlDocGetRootElement(doc)) == NULL) {
    xmlFreeDoc(doc);
    xmlCleanupParser();
    return 0;
  }
  if(strcmp(ptr->name, "config") != 0) {
    fprintf(stderr, "Config file not recognized\n");
    xmlFreeDoc(doc);
    xmlCleanupParser();
    return 0;
  }

  ptr = ptr->children;
  while(ptr != NULL) {
#define GSPEC(a, b) if(strcmp(ptr->name, (a)) == 0) { GSPEC1(ptr, (a), (b)); }
#define PSPEC(a, b, c) if(strcmp(ptr->name, (a)) == 0) { PSPEC1(ptr, (a), (b), (c)); }

    if(0) { } // XXX: continue
    else if(strcmp(ptr->name, "display") == 0) {
      if((pa = xmlGetProp(ptr, "sevenbit")) != NULL) {
        Optmap.n_disp = strtoul(pa, NULL, 0) > 0;
        xmlFree(pa);
      }
    }
    else GSPEC("skeld", &Optmap.f_skeld)
    else PSPEC("pager", &Optmap.p_pager, &Optmap.pa_pager)
*/
/*      byte *x, *y;
      if((x = xmlGetProp(ptr, "name")) != NULL &&
       (y = xmlGetProp(ptr, "value")) != NULL) {
        if(strcmp(x, "arcmeth") == 0) { OPT.nArcmeth = strtoul(y, NULL, 0); }
        else if(strcmp(x, "destgrp") == 0) { strclone(&OPT.sDestgrp, x); }
        else if(strcmp(x, "passlen") == 0) {
         OPT.nPasslen = strtoul(y, NULL, 0); }
      }
    }*/
/*
    ptr = ptr->next;
  }

  xmlFreeDoc(doc);
  xmlCleanupParser();
  return 1;
}
*/
int parse_options(int *argc, const char ***argv) {
  poptContext ctx;
  const char **args;
  int c, argk;
  char *cfg;

  struct poptOption options_table[] = {
    {NULL, 'a', POPT_ARG_NONE, NULL, 'a', "Display only 7-bit chars", NULL},
    {"config", 'c', POPT_ARG_STRING, &cfg, 'c',
     "Load alternate config file (ontop of Daxtraq's config.xml)", NULL},
    {NULL, 'g', POPT_ARG_NONE, NULL, 'g', "Go textgraphical", NULL},
    {NULL, 'h', POPT_ARG_NONE, NULL, 'h', "Show this help message", NULL},
    {"version", 'v', POPT_ARG_NONE, NULL, 'v', "Show version number and exit",
     NULL},
    POPT_AUTOHELP
    POPT_TABLEEND
  };

  ctx = poptGetContext(**argv, *argc, *argv, options_table, 0);
  while((c = poptGetNextOpt(ctx)) >= 0) {
    switch(c) {
      case 'a':
//        _textcon.cap |= HXTXD_CAP_7B;
        break;
/*      case 'c':
        if(!load_config(cfg)) {
         fprintf(stderr, "Parsing of %s unsuccessful\n", cfg); }
        break;*/
      case 'g':
        Optmap.graphical++;
        break;
      case 'h':
        poptPrintHelp(ctx, stdout, 0);
        exit(0);
        break;
      case 'v':
        printf("%s %s (%s)\n", Optmap.progn, Optmap.progv, Optmap.progd);
        exit(0);
        break;
    }
  }

  if(c < 0) {
    fprintf(stderr, "%s: %s\n", poptBadOption(ctx, 0), poptStrerror(c));
    return 0;
  }

  if((args = poptGetArgs(ctx)) != NULL) {
    argk = count_args(args);
    poptDupArgv(argk, args, argc, argv);
    poptFreeContext(ctx);
  }

  return 1;
}

static int count_args(const char **args) {
  int argk = 0;
  while(*args++ != NULL) { ++argk; }
  return argk;
}

//==[ End of file ]============================================================
